/*  Copyright (C) MOXA Inc. All rights reserved.

    This software is distributed under the terms of the
    MOXA License.  See the file COPYING-MOXA for details.
*/

#ifndef _MDM_API_
#define _MDM_API_

#ifdef __cplusplus
extern "C" {
#endif

#include <stdarg.h>
#include <stdlib.h>
#include <stdio.h>


/*	definition of the type of MDM program -- an argument of function mdm_start
*/
#define PROGRAM_TYPE_FORALL		0		/* not used */
#define PROGRAM_TYPE_DEVICE		1		/* a remote device */
#define PROGRAM_TYPE_GATEWAY	(1<<1)	/* a gateway */
#define PROGRAM_TYPE_CLIENT		(1<<2)	/* a client/tool */
#if !defined(_WIN32_WCE) && !defined(WIN32)
#define PROGRAM_TYPE_DLL_TEST	(1<<3)	/* test symbol relocation of Linux shared library */
#endif

typedef struct _MDMCONN	MDMCONN;

/*	get the number of elements in an array */
#define XtNumber(p) sizeof(p)/sizeof(p[0])

/*	A MDM program maintains a list of users. A user could be a device, 
	a gateway, or a client. */
typedef struct _MDMUSER
{
    char *nick;			/* a system-wide yet unique name */
    char *pass;			/* not used */
    MDMCONN *con;	/* connection bound to the user */
    unsigned int flags;	/* reserved for future use */
} MDMUSER;

/*	A connection could be a device, a gateway, or a client. Messages are 
	transmitted over it. */
struct _MDMCONN
{
	unsigned int type:8;	/* type of connection to the system */ 
	unsigned int binary_type:16;
	unsigned int silence:1;
	unsigned int xxx:7;
	void	*hndl;		/* pointer to the low level connection */
	MDMUSER    *user;		/* what user is  */
	char	*peer;		/* peer is talking */
	void	*data;		/* private data, careful use */
};


/*	start up a MDM Core in a program or thread with specified type(s)
	Inputs:
		<type> the type of program to start up
		<thread> 1 for thread and 0 for program
	Returns:
		The funtion returns 0 on success.
*/
int mdm_start(unsigned int type, unsigned int thread);

/*	check if the MDM Core is alive
	Inputs:
		none
	Returns:
		<int> 1 for alive and 0 for dead
*/
int mdm_alive(void);

/*	stop the MDM Core
	Inputs:
		None
	Returns:
		None
*/
void mdm_stop(void);

/*	This function is implemented in the UI/client. It is called when a message from a gateway arrives.
	Inputs:
		<con> the gateway
		<tag> message id
		<argc> the # of strings in the message
		<argv> the array of the strings
	Returns:
		none
*/
typedef void (*mdm_message_t) (MDMCONN *con, int tag, int argc, char **argv);

/*	This function is used to set a user-defined function for operating the messages delivered from gateways.
	Inputs:
		<msg> an user defined function for UI/client
	Returns:
		None
*/
void mdm_set_mngr_function(mdm_message_t msg);

/*	get the current gateway nick while in 2-tier architechture.
	Outputs:
		<location> gateway nick in <ip>:<port> format
	Returns:
		The funtion returns 0 on success.
*/
int mdm_get_gateway_location(char *location);

/*	split a message packet in char string format with space seperators into an array of string elements
	Inputs:
		<pkt> the message packet
		<count> the maxinum number of elements in the array
	Outputs:
		<av> point to the array of the string elements
	Returns:
		the number of string elements
*/
int split_line(char **av, int count, char *pkt);

/*	parse a message packet in char string format with space seperators
	Inputs:
		<pkt> point to the address of the message packet
	Outputs:
		<pkt> move the address to the next element
	Returns:
		point to the 1st element
*/
char* next_arg(char **pkt);

/*	send a formatted message over a one-hop connection. "one-hop" means UI/client<-->gateway or gateway<-->device.
	Inputs:
		<con> the connection
		<tag> message id
		<fmt> format the content of the message
	Returns:
		none
*/
void send_cmd(MDMCONN *con, unsigned int tag, const char *fmt, ...);

/*	send a formatted message over two-hop connections. "two-hop" means UI/client<-->gateway<-->device.
	Inputs:
		<con> the connection
		<nick> the name/id of the device
		<tag> message/id
		<fmt> format the content of the message
	Returns:
		none
*/
void send_peer(MDMCONN *con, char *nick, unsigned int tag, const char *fmt, ...);

/*	send a non-formatted message over a one-hop connection. 
	Inputs:
		<con> the connection
		<tag> message id
		<pkt> the message data
		<len> the length of the message
	Returns:
		none
*/
void send_cmd_data(MDMCONN *con, unsigned int tag, const char *pkt, unsigned int len);

/*	send a non-formatted message over two-hop connections. "two-hop" means UI/client<-->gateway<-->device.
	Inputs:
		<con> the connection
		<nick> the name/id of the device
		<tag> message/id
		<pkt> the message data
		<len> the length of the message
	Returns:
		none
*/
void send_peer_data(MDMCONN *con, char *nick, unsigned int tag, const char *pkt, unsigned int len);


#ifdef __cplusplus
}   
#endif

#endif /* _MDM_API_ */
